#!/usr/bin/env python3

"""
MPO publikuje každoročně zprávu Obnovitelné zdroje v roce RRRR se
statistikami výroby energií z OZE.
Modul pro scrapování odkazů na tyto pdf zprávy a jejich uložení ve formátu csv.

year,url
2020,https://www.mpo.cz/path/to/pdf
...,...
"""


import re
import csv
import requests
from bs4 import BeautifulSoup


HEADERS = {
    'user-agent': 'User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:84.0) Gecko/20100101 Firefox/84.0'
}

BASE_URL = 'https://www.mpo.cz'

START_URLS = [
    'https://www.mpo.cz/cz/energetika/statistika/obnovitelne-zdroje-energie/',
    'https://www.mpo.cz/cz/energetika/statistika/obnovitelne-zdroje-energie/archiv.htm',
]


def gather_post_urls(url, domain, headers):
    post_url_list = []

    r = requests.get(url, headers=headers)
    bs = BeautifulSoup(r.text, 'html.parser')
    a_tags_list = bs.find_all('a', text=re.compile('zdroje energie v roce'))

    for a in a_tags_list:
        post_url_list.append(domain + a.attrs['href'])

    return post_url_list


def gather_pdf_urls(post_url_list, domain, headers):
    report_url_list = []

    for post_url in post_url_list:
        r = requests.get(post_url, headers=headers)
        bs = BeautifulSoup(r.text, 'html.parser')

        download_url = domain + bs.find('a', {'class': 'download'}).attrs['href']
        year = re.search(re.compile('\d{4}'), post_url).group(0)
        report_url_list.append(
            {
                'year': year,
                'url': download_url,
            }
        )

    return report_url_list


def pdfs_urls_to_csv(
    filename='odkazy.csv', urls=START_URLS, base_url=BASE_URL, headers=HEADERS
):

    post_urls_list = []

    for url in urls:
        post_urls_list += gather_post_urls(url, base_url, headers)

    pdf_url_list = gather_pdf_urls(post_urls_list, base_url, headers)

    with open(filename, 'w', newline='') as csvf:
        writer = csv.DictWriter(csvf, fieldnames=['year', 'url'])
        writer.writeheader()
        writer.writerows(pdf_url_list)


def get_url_for_year(year: str):
    with open('odkazy.csv', newline='') as csvf:
        reader = csv.DictReader(csvf)
        for row in reader:
            if row['year'] == year:
                return row['url']
    return None


if __name__ == '__main__':
    pdfs_urls_to_csv()



from setuptools import find_packages, setup


with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name='mpo-oze',
    version='0.2.0',
    description='Parse reports covering renewables published by the Czech Ministry of Trade and Industry',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/jandolezal/mpo-oze',
    author='Jan Doležal',
    author_email='aswlk2zn2@mozmail.com',
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ],
    packages=['mpo'],
    install_requires=[
        'beautifulsoup4',
        'camelot-py[base]',
        'requests',
    ],
    entry_points={
        'console_scripts': [
            'mpo=mpo.__main__:main',
        ]
    }
)



#!/usr/bin/env python3

import csv
import re
import os

import camelot


def adjust_row(row):
    adjusted_row = []

    for cell in row:
        if ' ' in cell and re.fullmatch(re.compile(r'[^A-Za-z]+'), cell):
            adjusted_cell = cell.replace(' ', '')
            adjusted_row.append(adjusted_cell)
        elif '\n' in cell:
            adjusted_cell = cell.replace('\n', '')
            adjusted_row.append(adjusted_cell)
        else:
            adjusted_row.append(cell)

    return adjusted_row


def parse_pdf(pdf_url, year, pages='2-end'):
    print(f'Starting parsing the report for the year {year} from {pdf_url}')
    tables = camelot.read_pdf(pdf_url, pages=pages, suppress_stdout=True)

    if not os.path.exists(year):
        os.mkdir(year)

    # Write all tables directly to excel
    dirpath = os.path.abspath(year)
    excelpath = os.path.join(dirpath, year) + '.xlsx'
    tables.export(excelpath, f='excel')

    for table in tables:

        # If the parsing accuracy is low discard the table
        # These tables are usually not tables but figures (plots)
        if table.parsing_report["accuracy"] < -100.0:
            print(
                f'Skipping table page {table.parsing_report["page"]} order {table.parsing_report["order"]}'
            )
            continue

        # Get rid of spaces between digits for csv files
        adjusted_table = []

        for row in table.data:
            adjusted_table.append(adjust_row(row))

        page = str(table.parsing_report['page'])
        order = str(table.parsing_report['order'])

        csvname = f'{year}_page_{page}_table_{order}.csv'
        csvpath = os.path.join(dirpath, csvname)
        with open(csvpath, mode='w', newline='') as csvf:
            writer = csv.writer(csvf)
            writer.writerows(adjusted_table)

    print(f'Parsing completed.')


def main():
    with open('odkazy.csv', newline='') as csvf:
        reader = csv.DictReader(csvf)

        for row in reader:
            parse_pdf(row['url'], row['year'])


if __name__ == '__main__':
    main()



import argparse
import csv

from mpo import links, pdf, show


def main():
    parser = argparse.ArgumentParser(
        prog='mpo',
        description='parse czech renewable energy pdf reports',
    )
    parser.add_argument(
        'subcommand',
        choices=['links', 'pdf', 'show'],
        help='scrape links to reports, parse report or show report in a browser',
    )
    parser.add_argument(
        '-y',
        '--year',
        default='2020',
        help='select report year of interest (default: 2020)',
    )
    parser.add_argument(
        '-p',
        '--pages',
        default='2-end',
        help='select pages, e.g. 1,3-5,7-end (default: 2-end)',
    )
    args = parser.parse_args()

    # Get report urls
    if args.subcommand == 'links':
        links.pdfs_urls_to_csv()
    # Open report in a browser
    elif args.subcommand == 'show':
        report_url = links.get_url_for_year(args.year)
        show.open_report_page(report_url)
    # Parse report of interest
    elif args.subcommand == 'pdf':
        report_url = links.get_url_for_year(args.year)
        pdf.parse_pdf(pdf_url=report_url, year=args.year, pages=args.pages)
    else:
        print('Please use -h flag to see the options.')


if __name__ == '__main__':
    main()
